@@ -6,6 +6,7 @@ from __future__ import with_statement
 import sys
 import math
 import time
+import timeit
 import logging
 import warnings
 from datetime import datetime, timedelta
@@ -13,9 +14,10 @@ import collections
 
 from python_utils import converters
 
+import six
+
 from . import widgets
 from . import widgets as widgets_module  # Avoid name collision
-from . import six
 from . import base
 from . import utils
 
@@ -26,7 +28,7 @@ logger = logging.getLogger(__name__)
 class ProgressBarMixinBase(object):
 
     def __init__(self, **kwargs):
-        pass
+        self._finished = False
 
     def start(self, **kwargs):
         pass
@@ -35,7 +37,14 @@ class ProgressBarMixinBase(object):
         pass
 
     def finish(self):  # pragma: no cover
-        pass
+        self._finished = True
+
+    def __del__(self):
+        if not self._finished:  # pragma: no cover
+            try:
+                self.finish()
+            except Exception:
+                pass
 
 
 class ProgressBarBase(collections.Iterable, ProgressBarMixinBase):
@@ -75,14 +84,14 @@ class ResizableMixin(ProgressBarMixinBase):
         self.signal_set = False
         if term_width:
             self.term_width = term_width
-        else:
+        else:  # pragma: no cover
             try:
                 self._handle_resize()
                 import signal
                 self._prev_handle = signal.getsignal(signal.SIGWINCH)
                 signal.signal(signal.SIGWINCH, self._handle_resize)
                 self.signal_set = True
-            except Exception:  # pragma: no cover
+            except Exception:
                 pass
 
     def _handle_resize(self, signum=None, frame=None):
@@ -161,6 +170,9 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
         custom_len (function): Method to override how the line width is
             calculated. When using non-latin characters the width
             calculation might be off by default
+        max_error (bool): When True the progressbar will raise an error if it
+            goes beyond it's set max_value. Otherwise the max_value is simply
+            raised when needed
 
     A common way of using it is like:
 
@@ -210,7 +222,7 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
 
     def __init__(self, min_value=0, max_value=None, widgets=None,
                  left_justify=True, initial_value=0, poll_interval=None,
-                 widget_kwargs=None, custom_len=len,
+                 widget_kwargs=None, custom_len=len, max_error=True,
                  **kwargs):
         '''
         Initializes a progress bar with sane defaults
@@ -234,6 +246,7 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
                                  'value')
         self.min_value = min_value
         self.max_value = max_value
+        self.max_error = max_error
         self.widgets = widgets
         self.widget_kwargs = widget_kwargs or {}
         self.left_justify = left_justify
@@ -266,6 +279,7 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
         self.updates = 0
         self.end_time = None
         self.extra = dict()
+        self._last_update_timer = timeit.default_timer()
 
     @property
     def percentage(self):
@@ -350,6 +364,7 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
 
         '''
         self._last_update_time = time.time()
+        self._last_update_timer = timeit.default_timer()
         elapsed = self.last_update_time - self.start_time
         # For Python 2.7 and higher we have _`timedelta.total_seconds`, but we
         # want to support older versions as well
@@ -391,9 +406,6 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
 
     def default_widgets(self):
         if self.max_value:
-            self.widget_kwargs.setdefault(
-                'samples', max(10, self.max_value / 100))
-
             return [
                 widgets.Percentage(**self.widget_kwargs),
                 ' ', widgets.SimpleProgress(
@@ -464,7 +476,7 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
             if isinstance(widget, widgets.AutoWidthWidgetBase):
                 result.append(widget)
                 expanding.insert(0, index)
-            elif isinstance(widget, six.basestring):
+            elif isinstance(widget, six.string_types):
                 result.append(widget)
                 width -= self.custom_len(widget)
             else:
@@ -504,8 +516,8 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
         'Returns whether the ProgressBar should redraw the line.'
 
         if self.poll_interval:
-            delta = datetime.now() - self.last_update_time
-            poll_status = delta > self.poll_interval
+            delta = timeit.default_timer() - self._last_update_timer
+            poll_status = delta > self.poll_interval.total_seconds()
         else:
             poll_status = False
 
@@ -535,17 +547,19 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
             elif self.min_value <= value <= self.max_value:
                 # Correct value, let's accept
                 pass
-            else:
+            elif self.max_error:
                 raise ValueError(
                     'Value %s is out of range, should be between %s and %s'
                     % (value, self.min_value, self.max_value))
+            else:
+                self.max_value = value
 
             self.previous_value = self.value
             self.value = value
 
         minimum_update_interval = self._MINIMUM_UPDATE_INTERVAL
-        update_delta = time.time() - self._last_update_time
-        if update_delta < minimum_update_interval and not force:
+        delta = timeit.default_timer() - self._last_update_timer
+        if delta < minimum_update_interval and not force:
             # Prevent updating too often
             return
 
@@ -615,6 +629,7 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
             raise ValueError('Value out of range')
 
         self.start_time = self.last_update_time = datetime.now()
+        self._last_update_timer = timeit.default_timer()
         self.update(self.min_value, force=True)
 
         return self
